#include "CGraphicObject.h"
///////////////////////////////////////////////////////////////////
//  CGraphicObject.cpp:                                          //
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:          Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////

using namespace std;
/*
	constructor 
*/
CGraphicObject::CGraphicObject(char fileName[]){		

		m_bScreenWidth  = 750;
		m_bScreenHeight = 750;

		ReadFromFile(fileName);
		InitData();
}

/*
	deconstructor 
*/
CGraphicObject::~CGraphicObject(){
	int i;
	for( i=0;i<m_iRowNumber;i++)	
		delete [] m_bFileData[i];
	delete [] m_bFileData;
	ReleaseMem(true,true);
}

/*
	return the light source param
*/
ThreeDPoint CGraphicObject::GetLightParam(){
	return m_bLightSource;
}

/*
	set the screen width and height
*/
void CGraphicObject::SetScreenParam(int _w,int _h){
	m_bScreenWidth  = _w;
	m_bScreenHeight = _h;
}

/*
	load the data from a file
*/
int CGraphicObject::ReadFromFile(char fileName[]){
	int i,j;
	ifstream in(fileName);
	if(!in.good())
		return 0;
	in >> m_bFileLightSource.x>>m_bFileLightSource.y>>m_bFileLightSource.z;
	in >> m_iRowNumber >> m_iColNumber;
	m_bFileData = new double*[m_iRowNumber];
	for(i=0;i<m_iRowNumber;i++){
		m_bFileData[i] = new double[m_iColNumber];
	}
	for(i=0;i<m_iRowNumber;i++)
		for(j=0;j<m_iColNumber;j++){
			in >>m_bFileData[i][j];
		}	
	return 1;
}

/*
	init the graphic data for the first time
*/
void CGraphicObject::InitData(){
	ConvertData();
	InitPatch();
	UpdatePatch();
}

/*
	convert data to 3d points of the graphic object
*/
void CGraphicObject::ConvertData(){//file data ----> surface data
	int i,j;
	double _temp,_max = 0;
	m_tObjectData = new ThreeDPoint*[m_iRowNumber];
	for(i=0;i<m_iRowNumber;i++)
		m_tObjectData[i] = new ThreeDPoint[m_iColNumber];
	_temp = (1.6*m_bScreenWidth)/m_iColNumber; //-------------
	for(i=0;i<m_iRowNumber;i++)
		for(j=0;j<m_iColNumber;j++){
			_max =(m_bFileData[i][j]>_max)?m_bFileData[i][j]:_max;//--------
			m_tObjectData[i][j].x = (2*i-m_iColNumber)*_temp;
			m_tObjectData[i][j].z = (2*j-m_iRowNumber)*_temp;			
		}
	_temp = (1.6*m_bScreenHeight)/_max;
	for(i=0;i<m_iRowNumber;i++)
		for(j=0;j<m_iColNumber;j++)
			m_tObjectData[i][j].y = _temp*m_bFileData[i][j];
	//init Light
	m_bLightSource = m_bFileLightSource;
}

/*
	release the mem,
*/
void CGraphicObject::ReleaseMem(bool releaseObj,bool releasePatch){
	if (releaseObj && m_tObjectData){
		for(int i=0;i<m_iRowNumber;i++)
			delete[] m_tObjectData[i];
		delete[] m_tObjectData;
	}
  if (releasePatch && m_lPatchList){		
		for(int j=0;j<2*(m_iRowNumber-1);j++)
			delete[] m_lPatchList[j];
		delete[] m_lPatchList;
	}
}

/*
	return the row number
*/
int CGraphicObject::GetRowNumber(){
	return m_iRowNumber;
}

/*
	return the column number
*/
int CGraphicObject::GetColNumber(){
	return m_iColNumber;
}

/*
	build the normal and color for a single patch
*/
ThreeDPatch CGraphicObject::ProcessPatch(int i,int j){
	m_lPatchList[i][j].normal = MakeNormal(m_lPatchList[i][j]);
	m_lPatchList[i][j].color  = MakeNormalAngle(m_lPatchList[i][j]);
	return m_lPatchList[i][j];
}

/*
	build normal
*/
ThreeDPoint CGraphicObject::MakeNormal(ThreeDPatch _p){
	gaussel g;
	ThreeDPoint _result;
	long double **m, vec[3];
  m = g.createMat();
	m[0][0] = _p.a.x;	m[0][1] = _p.a.y;	m[0][2] = _p.a.z;
	m[0][3] = 1;	    m[1][3] = 1;	    m[2][3] = 1;
	m[1][0] = _p.b.x;	m[1][1] = _p.b.y;	m[1][2] = _p.b.z;
	m[2][0] = _p.c.x;	m[2][1] = _p.c.y;	m[2][2] = _p.c.z;
	g.GaussEl(m);	g.backSub(m);	g.sol(m,vec);	g.destroyMat(m);
	_result.x = vec[0];	_result.y = vec[1];	_result.z = vec[2];
	return _result;
}

/*
	build color for a patch
*/
double CGraphicObject::MakeNormalAngle(ThreeDPatch _p){
	double _light,_normal;
	_light = sqrt((pow(m_bLightSource.x,2)+pow(m_bLightSource.y,2)+ pow(m_bLightSource.z,2)));
	_normal = sqrt(pow(_p.normal.x,2)+pow(_p.normal.y,2)+pow(_p.normal.z,2));
	return innerProduct(_p.normal,m_bLightSource)/(_normal*_light);
}

/*
	Alloc memory for patch list
*/
void CGraphicObject::InitPatch(){
  m_lPatchList = new ThreeDPatch*[2*(m_iRowNumber-1)];
	for(int j=0;j<2*(m_iRowNumber-1);j++)
		m_lPatchList[j] =   new ThreeDPatch[m_iColNumber-1];
}

/*
	Reset the patch list after sth changed
*/
void CGraphicObject::UpdatePatch(){
	int count =0;
	for(int i=0;i<m_iRowNumber-1;i++)
		for(int j=0;j<m_iColNumber-1;j++){		
		//upper one
		m_lPatchList[i*2][j].a = m_tObjectData[i][j];
		m_lPatchList[i*2][j].b = m_tObjectData[i][j+1];
		m_lPatchList[i*2][j].c = m_tObjectData[i+1][j];		
		//next lower one
		m_lPatchList[i*2+1][j].a = m_tObjectData[i][j+1];
		m_lPatchList[i*2+1][j].b = m_tObjectData[i+1][j+1];
		m_lPatchList[i*2+1][j].c = m_tObjectData[i+1][j];
	}
}
/*
	inner Product
*/
double CGraphicObject::innerProduct(ThreeDPoint P,ThreeDPoint Q)
{
	return (P.x*Q.x)+(P.y*Q.y)+(P.z*Q.z);
}
/*
	zoom the obect
*/
void CGraphicObject::DoZoom(double factor){
	for(int i=0;i<m_iRowNumber;i++)
		for(int j=0;j<m_iColNumber;j++){
			m_tObjectData[i][j].x *= factor;
			m_tObjectData[i][j].y *= factor;
			m_tObjectData[i][j].z *= factor;
		}
	UpdatePatch();	
}
/*
	translate x of object
*/
void CGraphicObject::DoTranslationX(double delta){
	for(int i=0;i<m_iRowNumber;i++)
		for(int j=0;j<m_iColNumber;j++){
			m_tObjectData[i][j].x += delta;
		}
	UpdatePatch();
}
/*
	translate y of object
*/
void CGraphicObject::DoTranslationY(double delta){
	for(int i=0;i<m_iRowNumber;i++)
		for(int j=0;j<m_iColNumber;j++){
			m_tObjectData[i][j].y += delta;
		}
	UpdatePatch();
}
/*
	translate  z of object
*/
void CGraphicObject::DoTranslationZ(double delta){
	for(int i=0;i<m_iRowNumber;i++)
		for(int j=0;j<m_iColNumber;j++){
			m_tObjectData[i][j].z += delta;
		}
	UpdatePatch();
}
/*
	Rotate viewpoint along y axis
*/
void CGraphicObject::DoRotateObserverAzimuth(double angle,ThreeDPoint* m_bScreenCenterPoint,ThreeDPoint* m_bObservePoint){	
	double _newX, _newZ;
	_newX = m_bScreenCenterPoint->x*cos(angle)+m_bScreenCenterPoint->z*sin(angle);
	_newZ = -m_bScreenCenterPoint->x*sin(angle)+m_bScreenCenterPoint->z*cos(angle);
	m_bScreenCenterPoint->x = _newX;
	m_bScreenCenterPoint->z = _newZ;

	_newX = m_bObservePoint->x*cos(angle)+m_bObservePoint->z*sin(angle);
	_newZ = -m_bObservePoint->x*sin(angle)+m_bObservePoint->z*cos(angle);
	m_bObservePoint->x = _newX;
	m_bObservePoint->z = _newZ;
	UpdatePatch();
}

/*
	Rotate viewpoint along x axis
*/
void CGraphicObject::DoRotateObserverElevation(double angle,ThreeDPoint* m_bScreenCenterPoint,ThreeDPoint* m_bObservePoint){
	double _newY, _newZ;
	_newY = m_bScreenCenterPoint->y*cos(angle)+m_bScreenCenterPoint->z*sin(angle);
	_newZ = -m_bScreenCenterPoint->y*sin(angle)+m_bScreenCenterPoint->z*cos(angle);
	m_bScreenCenterPoint->y = _newY;
	m_bScreenCenterPoint->z = _newZ;

	_newY = m_bObservePoint->y*cos(angle)+m_bObservePoint->z*sin(angle);
	_newZ = -m_bObservePoint->y*sin(angle)+m_bObservePoint->z*cos(angle);
	m_bObservePoint->y = _newY;
	m_bObservePoint->z = _newZ;
	UpdatePatch();
}

/*
	Rotate light along y axis
*/
void CGraphicObject::DoRotateLightAzimuth(double angle){
	double _newX, _newZ;
	_newX = m_bLightSource.x*cos(angle)+m_bLightSource.z*sin(angle);
	_newZ = -m_bLightSource.x*sin(angle)+m_bLightSource.z*cos(angle);
	m_bLightSource.x = _newX;
	m_bLightSource.z = _newZ;
}
/*
	Rotate light along x axis
*/
void CGraphicObject::DoRotateLightElevation(double angle){
	double _newY, _newZ;
	_newY = m_bLightSource.y*cos(angle)+m_bLightSource.z*sin(angle);
	_newZ = -m_bLightSource.y*sin(angle)+m_bLightSource.z*cos(angle);
	m_bLightSource.y = _newY;
	m_bLightSource.z = _newZ;

}
/*
	reset 
*/
void CGraphicObject::DoReset(){
	ReleaseMem(true,true);
	InitData();
}

//--------------------< test stub >---------------------------

#ifdef TEST_CGRAPHICOBJECT

void main() {

	CGraphicObject* p;
	p = new CGraphicObject("surf40.dat");
	p.InitData();
	p.DoTranslationX(100);
	p.DoTranslationY(100);
	p.DoTranslationZ(100);
	p.DoRotateObserveAzimuth(10);
	p.DoRotateObserveElevation(10);
	p.DoRotateLightAzimuth(10);
	p.DoRotateLightElevation(10);
	delete p;

	return;
}

#endif



