#ifndef CGRAPHICOBJECT_H
#define CGRAPHICOBJECT_H
///////////////////////////////////////////////////////////////////
//	CGraphicObject.h                                             //
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:          Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////
/*

    Module Operations
    =================
		The CGraphicObject define and implemente the basic operations
		for a graphic object;

    Public Interface
    ================

		ThreeDPoint GetLightParam();		
		-------return the light source Param
		void		SetScreenParam(int _w,int _h);
		-------Set the Width and height of screen

		int			GetRowNumber();
		-------Get the Row number of the data 
		int			GetColNumber();
		-------Get the Colnum number of the data 

		void	 DoReset();	
		-------Reset the graphic 
		
		void		DoZoom(double factor);
		-------Zoom the object

		void		DoTranslationX(double delta);	
		-------Move the x of object 
		void		DoTranslationY(double delta);
		-------Move the y of object 
		void		DoTranslationZ(double delta);
		-------Move the z of object 

		void		DoRotateObserverAzimuth(double angle,ThreeDPoint* m_bScreenCenterPoint,ThreeDPoint* m_bObservePoint);
		-------Rotate the viewpoint 
		void		DoRotateObserverElevation(double angle,ThreeDPoint* m_bScreenCenterPoint,ThreeDPoint* m_bObservePoint);
		-------Rotate the viewpoint
		
		void		DoRotateLightAzimuth(double angle);
		-------Rotate the light
		void		DoRotateLightElevation(double angle);
		-------Rotate the light

		ThreeDPatch	ProcessPatch(int i,int j);
		-------Build the normal and color for each patch
*/

//
///////////////////////////////////////////////////////////////
//  build process                                            //
///////////////////////////////////////////////////////////////
//  required files:                                          //
//    gaussel.h, gaussel.cpp                                 //
//    CGraphicObject.h CGraphicObject.cpp										 //
//                                                           //
//  compiler command line invocation:                        //
//    cl -GX -DTEST_CGRAPHICOBJECT CGraphicObject.cpp        // 
//														gaussel.cpp                    //
///////////////////////////////////////////////////////////////
/*
    Maintenance History
    ===================
    ver 1.0 

*/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <direct.h>
#include <stdlib.h>
#include <math.h>
#include <string>
#include "gaussel.h"

#define PI	3.141592653589793238


typedef struct ThreeDPoint{
	double x,y,z;		
}ThreeDPoint;

typedef struct ThreeDPatch{
	ThreeDPoint a,b,c;
	double			color;
	ThreeDPoint	normal; 
}ThreeDPatch;

typedef struct PolarPoint{
	double pointO,angleA,angleE;
}PolarPoint;

class CGraphicObject{
private:
	//private data

	//data read from the file
	double**			m_bFileData;
	//3D point list of the graphic object
	ThreeDPoint**	m_tObjectData;	
	//3D Patch list of the graphic object
	ThreeDPatch**  m_lPatchList;
	//light source read from file
	ThreeDPoint		m_bFileLightSource;		
	//light source used for projection
	ThreeDPoint		m_bLightSource;		

	//row number
	int						m_iRowNumber;
	//column number
	int						m_iColNumber;
	//screen width	
	int						m_bScreenWidth;
	//screen height
	int						m_bScreenHeight;

	//private function
	//load data from a file
	int						ReadFromFile(char fileName[]);
	//covert data to 3d data
	void					ConvertData();	
	//generate the patch list
	void					InitPatch();	
	//re generate the patch list after user change sth
	void					UpdatePatch();	
	//caculate the normal for a patch
	ThreeDPoint		MakeNormal(ThreeDPatch _p);
	//caculate the color for a patch
	double 				MakeNormalAngle(ThreeDPatch _p);			
	//release the memory
	void					ReleaseMem(bool releaseObj,bool releasePatch);
	//init the data
	void					InitData();
public :	
	CGraphicObject(char fileName[]);//constructor from a file name	
	~CGraphicObject();		

	ThreeDPoint GetLightParam();	
	
	void		SetScreenParam(int _w,int _h);
	int			GetRowNumber();
	int			GetColNumber();
  
	
	void		DoZoom(double factor);
	void		DoTranslationX(double delta);	
	void		DoTranslationY(double delta);
	void		DoTranslationZ(double delta);
	void		DoRotateObserverAzimuth(double angle,ThreeDPoint* m_bScreenCenterPoint,ThreeDPoint* m_bObservePoint);
	void		DoRotateObserverElevation(double angle,ThreeDPoint* m_bScreenCenterPoint,ThreeDPoint* m_bObservePoint);
	void		DoRotateLightAzimuth(double angle);
	void		DoRotateLightElevation(double angle);
	void		DoReset();	

	ThreeDPatch	ProcessPatch(int i,int j);
	double				innerProduct(ThreeDPoint P,ThreeDPoint Q);

};


#endif