#include "CGraphicViewer.h"
#include "resource.h"
#include <windows.h>
///////////////////////////////////////////////////////////////////
//  CGraphicViewer.cpp:                                          //
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:          Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////

using namespace std;
/*
	constructor
*/
CGraphicViewer::CGraphicViewer(){
	m_pObject = NULL;
	InitViewer();
}
/*
	deconstructor
*/
CGraphicViewer::~CGraphicViewer(){
	if (m_pObject)
		delete m_pObject;
}

/*
	set the init param
*/
void CGraphicViewer::InitViewer(){
	//set the observer point and screen stuff.
	m_bScreenWidth  = 750;	m_bScreenHeight = 750;

	m_bObservePoint.x = 0;	m_bObservePoint.y = 0;
	m_bObservePoint.z = 4*m_bScreenWidth+(0.5*3.2*m_bScreenWidth);

	m_bScreenCenterPoint.x = 0;	m_bScreenCenterPoint.y = 0;
	m_bScreenCenterPoint.z = 3*m_bScreenWidth+(0.5*3.2*m_bScreenWidth);

	m_bObjectRotateAngleA=0;
	m_bObjectRotateAngleE=0;
}

/*
	load a graphic object from file and attach it
*/
void CGraphicViewer::AttachObject(char objectFileName[]){
	if (m_pObject)
		delete m_pObject;	
	m_pObject = new CGraphicObject(objectFileName);
	m_pObject->SetScreenParam(m_bScreenWidth,m_bScreenHeight);
}

/*
	detach a graphic object
*/
void CGraphicViewer::DetachObject(){
	if(m_pObject)
		delete m_pObject;
	m_pObject = NULL;
}

/*
	return the object attached
*/
CGraphicObject*	CGraphicViewer::GetGraphicObject(){
	return m_pObject;
}

/*
	//re Draw the Graphic according to command
*/
void CGraphicViewer::RePaintObject(HWND hwnd){
	int          i;
	int _row,_col,_id=0; 
	if(!m_pObject)
		return;
	_row = m_pObject->GetRowNumber();
	_col = m_pObject->GetColNumber();

	PAINTSTRUCT  ps;			// structure to hold paint info
	HDC			 hdc;			// handle to device context
	hdc = BeginPaint(hwnd, &ps);	
	SelectObject(hdc,CreatePen(PS_NULL,1,0));
	for(int j=0;j<_col-1;j++)
		for( i=0;i<2*(_row-1);i++){ 
			ThreeDPatch  _p;
			_p= m_pObject->ProcessPatch(i,j);	
			DrawAPatch(hdc,_p);				
	}
	EndPaint(hwnd, &ps);
}

/*
	draw a patch on screen
*/
void CGraphicViewer::DrawAPatch(HDC hdc,ThreeDPatch p){
	//just draw the PloyGon
	static POINT patch[3];
	ThreeDPatch newPatch;
	newPatch = ProjectAPatchToScreen(p);
	patch[0].x = newPatch.a.x;
	patch[0].y = newPatch.a.y;
	patch[1].x = newPatch.b.x;
	patch[1].y = newPatch.b.y;
	patch[2].x = newPatch.c.x;
	patch[2].y = newPatch.c.y;
	double R = 255*fabs(p.color);
	double G = 0;
	double B = 255*fabs(p.color);
	HBRUSH hb = CreateSolidBrush(RGB(0,R,0));
	SelectObject(hdc,hb);
	Polygon(hdc,patch,3);
	DeleteObject(hb);
}

/*
	project a patch to screen
*/
ThreeDPatch CGraphicViewer::ProjectAPatchToScreen(ThreeDPatch _patch){
	ThreeDPatch _r;
	_r.a = ProjectAPointToScreen(_patch.a);
	_r.b = ProjectAPointToScreen(_patch.b);
	_r.c = ProjectAPointToScreen(_patch.c);
	return _r;
}

/*
	project a point to screen
*/
ThreeDPoint CGraphicViewer::ProjectAPointToScreen(ThreeDPoint _t){
	//project a point on screen ,should have return value	
	ThreeDPoint newPoint;
	
	double lambda = (m_pObject->innerProduct(m_bScreenCenterPoint,m_bScreenCenterPoint)-m_pObject->innerProduct(m_bScreenCenterPoint,m_bObservePoint))/(m_pObject->innerProduct(m_bScreenCenterPoint,_t)-m_pObject->innerProduct(m_bScreenCenterPoint,m_bObservePoint));
	newPoint.x = (lambda*_t.x+(1-lambda)*m_bObservePoint.x);		//	x coord on screen
	newPoint.y = (lambda*_t.y+(1-lambda)*m_bObservePoint.y);		//	y coord on screen
	newPoint.z = (lambda*_t.z+(1-lambda)*m_bObservePoint.z);		//  z coord which is discarded
	PolarPoint point  = ToPolar(newPoint);
	PolarPoint pointObserve = ToPolar(m_bObservePoint);	
	//	mapping the screen center to center of rotational axis	
	point.angleA = point.angleA+(PI/2)- pointObserve.angleA;
	point.angleE = point.angleE-pointObserve.angleE;	

	m_bObservePoint = FromPolar(pointObserve);
	newPoint = FromPolar(point);	
	//	snapping the image on to the screen center
	newPoint.x = newPoint.x+m_bScreenWidth/2;	newPoint.y = m_bScreenWidth/2-newPoint.y;
	return newPoint;		//	return the projection coordinates
}

/*
	Polar to rectangular 
*/
ThreeDPoint CGraphicViewer::FromPolar(PolarPoint pPoint){
	ThreeDPoint _t;
	_t.x = pPoint.pointO*cos(pPoint.angleA)*cos(pPoint.angleE);
	_t.y = pPoint.pointO*sin(pPoint.angleE);
	_t.z = pPoint.pointO*cos(pPoint.angleE)*sin(pPoint.angleA);
	return _t;
}
/*
	rectangular to polar
*/
PolarPoint	CGraphicViewer::ToPolar(ThreeDPoint tPoint){
	PolarPoint _p;
	_p.pointO = sqrt(pow(tPoint.x,2)+pow(tPoint.y,2)+pow(tPoint.z,2));	
	_p.angleA = atan2(tPoint.z,tPoint.x);
	_p.angleE = atan2(tPoint.y,(sqrt(pow(tPoint.x,2)+pow(tPoint.z,2))));
	return _p;
}

/*
	remember the which menuitem has been clicked
*/
void CGraphicViewer::SetCommand(WPARAM _w,HWND hwnd){	
	m_wCommand = _w;
	if (_w== _Reset){
		DoReset();
		InvalidateRect(hwnd,NULL,TRUE);	
	}
	if (_w == Demo)//oh, demo it!!
		AllDemo(hwnd);
	return;
}

/*
	Reset the viewer
*/
void CGraphicViewer::DoReset(){
	InitViewer();
	m_pObject->DoReset();			
}

/*
	display an info on the center of screen 
*/
void CGraphicViewer::ShowText(HWND hwnd,char staticText[])
{
	PAINTSTRUCT		ps;			  // structure to hold paint info
	HDC						hdc;			// handle to device context

	hdc = BeginPaint(hwnd, &ps);
	SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT));
	SetTextColor (hdc, RGB(0,0,255));
	TextOut(hdc,250,200,staticText,strlen(staticText));
	EndPaint (hwnd, &ps);
	Sleep(10);
}

/*
	demo all the functions
*/
void CGraphicViewer::AllDemo(HWND hwnd){
	if (!m_pObject){
		ShowText(hwnd,"Please Open A file first");	
		Sleep(1000);
		return;
	}
	DoReset();
	RePaintObject(hwnd);
	InvalidateRect(hwnd,NULL,TRUE);	

	ShowText(hwnd,"Demo TranslationX");	
	DoDemo(TranslationX,hwnd);	
	ShowText(hwnd,"Demo TranslationY");
	DoDemo(TranslationY,hwnd);	
	ShowText(hwnd,"Demo TranslationZ");
	DoDemo(TranslationZ,hwnd);
	
	ShowText(hwnd,"Demo RotateObserverAzimuth");
	DoDemo(RotateObserverAzimuth,hwnd);	
	ShowText(hwnd,"Demo RotateObserverElevation");
	DoDemo(RotateObserverElevation,hwnd);
	
	ShowText(hwnd,"Demo RotateLightAzimuth");
	DoDemo(RotateLightAzimuth,hwnd);	
	ShowText(hwnd,"Demo RotateLightElevation");
	DoDemo(RotateLightElevation,hwnd);
	
	ShowText(hwnd,"Demo Zoom");
	DoDemo(Zoom,hwnd);
	
	ShowText(hwnd,"Demo Finished!");
	Sleep(200);
	DoReset();
}

/*
	demo a single fuction given by _w;
*/
void CGraphicViewer::DoDemo(WPARAM _w,HWND hwnd){
	int i,demoNumber=5;
	int sleepNumber=50;
	int deltaT =-400;
	m_wCommand = _w;
	Sleep(30*sleepNumber);
	DoReset();	
	for(i=0;i<demoNumber;i++){	
		//demo increase 
		DoCommand(2);
		RePaintObject(hwnd);
		InvalidateRect(hwnd,NULL,TRUE);	
		Sleep(sleepNumber*2);//0);
	}
	DoReset();	
	for(i=0;i<demoNumber;i++){	
		//demo decrease
		DoCommand(3);
		RePaintObject(hwnd);
		InvalidateRect(hwnd,NULL,TRUE);	
		Sleep(sleepNumber*2);//0);
	}
}

/*
	chech the range of rotation angle
*/
double CGraphicViewer::AngleRangeCheck(double _d, double* _p){
	double deltaR = _d;
	if((*_p+deltaR)>MAX_ANGLE){
		deltaR = MAX_ANGLE -*_p;
		*_p= MAX_ANGLE;
	}else if((*_p+deltaR)< -MAX_ANGLE){
		deltaR = -MAX_ANGLE -*_p;
		*_p= -MAX_ANGLE;
	}else *_p+=deltaR;
	return deltaR;
}

/*
	do sth when user click button
	
*/
void CGraphicViewer::DoCommand(int  m){
	int deltaT;
	double deltaR,deltaZ;	
	if (m==0){//left button
		deltaT = -100;		deltaR = -(5.0/180.0)*PI;		deltaZ = 0.9;
	}else if(m==1){//right button
		deltaT= 100;		deltaR= (5.0/180.0)*PI;		deltaZ= 1.1;
	}else if(m==2){ //demo increase
		deltaT= 400;		deltaR= 2*(5.0/180.0)*PI;		deltaZ= 1.1*1.1;
	}else if(m==3){ //demo decrease
		deltaT= -400;		deltaR= -2*(5.0/180.0)*PI;		deltaZ= 0.9*0.9;
	}	
	switch(m_wCommand){
		case TranslationX:
			m_pObject->DoTranslationX(deltaT);
			return;
		case TranslationY:
			m_pObject->DoTranslationY(deltaT);
			return;
		case TranslationZ:
			m_pObject->DoTranslationZ(deltaT);
			return;
		case RotateObserverAzimuth:
			//range checking
			deltaR = AngleRangeCheck(deltaR,&m_bObjectRotateAngleA);
			m_pObject->DoRotateObserverAzimuth(deltaR,&m_bScreenCenterPoint,&m_bObservePoint);
			return;			
		case RotateObserverElevation:
			//range checking
			deltaR = AngleRangeCheck(deltaR,&m_bObjectRotateAngleE);
			m_pObject->DoRotateObserverElevation(deltaR,&m_bScreenCenterPoint,&m_bObservePoint);
			return;
		case RotateLightAzimuth:
			m_pObject->DoRotateLightAzimuth(deltaR);
			return;
		case RotateLightElevation:
			m_pObject->DoRotateLightElevation(deltaR);
			return;
		case Zoom:
			m_pObject->DoZoom(deltaZ);
			return;
    default:
			return;
	}
}



//--------------------< test stub >---------------------------

#ifdef TEST_CGRAPHICVIEWER
void main() {

	CGraphicViewer* p;
	p = new CGraphicViewer();
	p.AttachObject("surf40.dat");
	p.AllDemo();
	p.DetachObject();
	delete p;


}

#endif



