#ifndef CGRAPHICVIEWER_H
#define CGRAPHICVIEWER_H
///////////////////////////////////////////////////////////////////
//	CGraphicViewer.h                                             //
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:          Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////
/*

    Module Operations
    =================
		The CGraphicObject define and implemente the basic operations
		for a graphic object;

    Public Interface
    ================
		void				InitViewer();	
		-------Init the viewer when user open  a file or reset
		void				AttachObject(char objectFileName[]);
		-------load a GraphicObject from a file and attch it 
		void				DetachObject();				
		-------Detach a graphic object
		void				RePaintObject(HWND hwnd);
		-------repaint after sth changed
		void				SetCommand(WPARAM _w,HWND hwnd);
		-------remember which menuitem has been clicked
		void				DoCommand(int _m);
		-------do the command given by user
		void				AllDemo(HWND hwnd);
		-------automatically demo all the funtions
		void				DoReset();
		-------reset the viewer
		CGraphicObject*	GetGraphicObject();
		-------return the object attached to this viewer
*/

//
///////////////////////////////////////////////////////////////
//  build process                                            //
///////////////////////////////////////////////////////////////
//  required files:                                          //
//    gaussel.h, gaussel.cpp                                 //
//    CGraphicObject.h CGraphicObject.cpp										 //
//    CGraphicViewer.h CGraphicViewer.cpp										 //
//                                                           //
//  compiler command line invocation:                        //
//    cl -GX -DTEST_CGRAPHICOBJECT CGraphicObject.cpp        // 
//														gaussel.cpp                    //
///////////////////////////////////////////////////////////////

/*
    Maintenance History
    ===================
    ver 1.0 

*/


#include <windows.h>
#include "CGraphicObject.h"


#define MAX_ANGLE (45.0/180)*PI

class CGraphicViewer{
private:
	ThreeDPoint			m_bObservePoint;
	ThreeDPoint			m_bScreenCenterPoint;				
	CGraphicObject*	m_pObject;
	
	int							m_bScreenWidth;
	int							m_bScreenHeight;
	double					m_bObjectRotateAngleA;
	double					m_bObjectRotateAngleE;
	WPARAM					m_wCommand;


	//private function
	//just draw a patch 
	void				DrawAPatch(HDC hdc,ThreeDPatch p);	
	//chech the range of angle +/-45 degree 
	double			AngleRangeCheck(double _d, double* _p);
	//project a patch to the screen
	ThreeDPatch ProjectAPatchToScreen(ThreeDPatch _patch);
	//project a point to the screen
	ThreeDPoint ProjectAPointToScreen(ThreeDPoint _t);
		
	ThreeDPoint FromPolar(PolarPoint pPoint);
	PolarPoint	ToPolar(ThreeDPoint tPoint);
	void				ShowText(HWND hwnd,char staticText[]);
	void				DoDemo(WPARAM _w,HWND hwnd);	
public :
	CGraphicViewer();
	~CGraphicViewer();
	void				InitViewer();	
	void				AttachObject(char objectFileName[]);
	void				DetachObject();				
	void				RePaintObject(HWND hwnd);
	void				SetCommand(WPARAM _w,HWND hwnd);
	void				DoCommand(int _m);
	void				AllDemo(HWND hwnd);
	void				DoReset();
	CGraphicObject*	GetGraphicObject();

};


#endif