///////////////////////////////////////////////////////////////////
//  DataViz.cpp:                                                 //
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:          Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////

/*
    Program Operations
    ==================
	This program read data from a file and generate a graphic object and 
	display it on screen.



	Menu is the user interface and provide:
	File  
				File->Open : open a file from hard disk and display it.
				File->Exit : close the program

	Operations
				Operations->TranslationX : move the object left or right
				Operations->TranslationY : move the object up   or down
				Operations->TranslationZ : move the object away from user or closer to user
				
				Operations->RotateObserverAzimuth   : rotate the viewpoint along y axis
				Operations->RotateObserverElevation : rotate the viewpoint along x axis

				Operations->RotateLightAzimuth      : rotate the light along y axis
				Operations->RotateLightElevation    : rotate the light along x axis
			  	
				Operations->Zoom  : zoom the object
				Operations->Reset : reset

	Additional
				Additional->Demo : After you open a file, it will demo all the functions 

    Main tenance Information
    =======================

      Build Process:
      --------------
        Required Files: DataViz.cpp, gaussel.cpp,CGraphicViewer.cpp,CGraphicObject.cpp, menu.rc,
						gaussel.h,resource.h,CGraphicViewer.h,CGraphicObject.h

        Building Executable:
          - File\New\Project\Win32 Application
          - Project\Add to Project\Files\DataViz.cpp
				  - Project\Add to Project\Files\gaussel.cpp
				  - Project\Add to Project\Files\CGraphicObject.cpp
					- Project\Add to Project\Files\CGraphicViewer.cpp
				  - Project\Add to Project\Files\gaussel.h
				  - Project\Add to Project\Files\CGraphicObject.h
					- Project\Add to Project\Files\CGraphicViewer.h
					- Project\Add to Project\Files\menu.rc
					- Project\Add to Project\Files\resource.h

          - compile, link, and execute
    
      Maintenance History:
      --------------------
      ver 1.0 
        - first release													   */

//---------------------------------------------------------------------------

#include <windows.h>
#include "resource.h"
#include "CGraphicViewer.h"

using namespace std;


#define  MAXWRITE  256
char	 _fileName[MAXWRITE] ="surf40.dat";

static CGraphicViewer* m_pGraphic;


LRESULT CALLBACK WndProc  (HWND, UINT, WPARAM, LPARAM);

/*
		entry point the program
		WinMain();
*/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{	
	HWND  hwnd;  
	MSG   msg;   
	// build window class structure
	WNDCLASSEX  wndclass;
	static char szAppName[] = "Data Visulization by Jianxiang Jin";
	wndclass.cbSize         = sizeof (wndclass);
	wndclass.style          = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc    = WndProc;
	wndclass.cbClsExtra     = 0;	wndclass.cbWndExtra     = 0;
	wndclass.hInstance      = hInstance;
	wndclass.hIcon          = LoadIcon (NULL, IDI_APPLICATION);
	wndclass.hCursor        = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName   = MAKEINTRESOURCE(IDR_MENU1);
	wndclass.lpszClassName  = szAppName;
	wndclass.hIconSm        = LoadIcon (NULL, IDI_APPLICATION);
	// register window class 
	RegisterClassEx (&wndclass);
	// create main window at specified location with specified size  
	hwnd = CreateWindow (szAppName, szAppName,WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,0, 0, 750, 750,NULL, NULL, hInstance, NULL);
	// display window
	ShowWindow (hwnd, iCmdShow);
	UpdateWindow (hwnd);
	// start message loop
	while (GetMessage (&msg, NULL, 0, 0))
	{
		TranslateMessage (&msg);
		DispatchMessage (&msg);
	}
	return msg.wParam;
}

/*
		Call back function of the dialog for open a file
		OpenFileDlg();
*/
BOOL CALLBACK OpenFileDlg(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam){
	switch(uMsg){
	case WM_INITDIALOG:
		SetDlgItemText(hDlg,IDC_EDIT1,_fileName);
		return TRUE;
	case WM_COMMAND:
		switch(LOWORD(wParam)){
		case IDC_EDIT1:	
			GetDlgItemText( hDlg, IDC_EDIT1, _fileName,MAXWRITE);
			break;		
    // click ok
		case IDOK:		
			EndDialog(hDlg,0);
			break;
		//click cancel
		case IDCANCEL:	
			EndDialog(hDlg,1);
			break;
		}
		return TRUE;
	}
	return FALSE;
}

/*
	Call back function of about dialog
	AboutDlg();
	
*/
BOOL CALLBACK AboutDlg(	HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam  )
{
	switch(uMsg)
	{
	case WM_INITDIALOG:
		return TRUE;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDOK:
			EndDialog(hDlg,0);
			break;
		}
		return TRUE;
	}
	return FALSE;
}

/*
	Call back function to handle all the windows message

*/
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	static HINSTANCE hInstance;				// handle to module	
	static RECT      smallRect,rect;		// paint mouse coordinates here

	switch (iMsg)	{	
	case WM_CREATE:
		//initialWindow(hwnd,rect,smallRect);
		m_pGraphic = new CGraphicViewer();
		return 0;
	case WM_COMMAND : //handle file dialog and menu		
		switch(wParam){
			case OpenFile:{
				//show the dialog 				
				//if click ok, then open the file, and attach graphic object to the window
				if(!DialogBox(hInstance,MAKEINTRESOURCE(IDD_DIALOG1),hwnd,OpenFileDlg))				
					m_pGraphic->AttachObject(_fileName);
				InvalidateRect(hwnd,NULL,TRUE);					
				return 0;}
			case Contents:
				DialogBox(hInstance,MAKEINTRESOURCE(IDD_DIALOG2),hwnd,AboutDlg);
				return 0;
			case About:
				DialogBox(hInstance,MAKEINTRESOURCE(IDD_DIALOG3),hwnd,AboutDlg);
				return 0;
			case Exit:
				PostQuitMessage(0);
				return 0;
			default:
				//let m_pGraphic remember which menuitem has been clicked
				m_pGraphic->SetCommand(wParam,hwnd);
				return 0;
		}
	// use click the left button,rebuild according to the command and display
	case WM_LBUTTONDOWN:
		m_pGraphic->DoCommand(0);
		InvalidateRect(hwnd,NULL,TRUE);	
		return 0;
	// use click the right button,rebuild according to the command and display
	case WM_RBUTTONDOWN:
		m_pGraphic->DoCommand(1);
		InvalidateRect(hwnd,NULL,TRUE);	
		return 0;
	//repaint
  case WM_PAINT :
		m_pGraphic->RePaintObject(hwnd);
		return 0;
	case WM_DESTROY:
		delete m_pGraphic;
    PostQuitMessage (0);
    return 0;	
	}
  return DefWindowProc (hwnd, iMsg, wParam, lParam);
}
