
#ifndef GAUSSEL_H
#define GAUSSEL_H
///////////////////////////////////////////////////////////////
//                                                           //
//  Gaussel.h   --  Gaussian Elimination                     //
//  ver 1.2         only for 3 by 4 augmented matrix         //
//                                                           //
//  Language:       Visual C++, ver 6.0,  used like C        //
//  Platform:       Micron Dual Pentium Pro 200, win NT 4.0  //
//  Application:    Prototype for CSE691, project #2         //
//  Author:         Jim Fawcett, CST 2-187, x3948            //
//                  fawcett@ecs.syr.edu                      //
//                                                           //
///////////////////////////////////////////////////////////////
/*

    Module Operations
    =================
    The gaussel module implements gaussian elimination and solution
    of equations for a system of 3 equations in 3 unknowns.  It is
    a server for graphics processing implemented in dataviz.cpp.
    The computations carried out here could be generalized for 
    square systems of equations of arbitrary size.  Also, the pro-
    cessing implemented here would fit naturally into a vector
    matrix class library.

    This header is used for both gaussel.c and gaussel.cpp, since
    the C++ version is written in the C style using only global
    functions.

    Public Interface
    ================
    double **mat = createMat();     // build a 3 by 4 matrix which
                                    // will hold coefficients of
                                    // equations and right hand side
                                    // constants (in last column)
    GaussEl(mat);                   // reduces mat to echelon form
    backSub(mat);                   // computes solution and stores
                                    // in last column of mat
    sol(mat,vec);                   // extracts solution and stores
                                    // in vec, an array of doubles
    destroyMat(mat);                // releases mat memory allocation

*/

//
///////////////////////////////////////////////////////////////
//  build process                                            //
///////////////////////////////////////////////////////////////
//  required files:                                          //
//    gaussel.h, gaussel.cpp                                 //
//                                                           //
//  compiler command line invocation:                        //
//    cl -GX -DTEST_GAUSSEL gaussel.cpp                      //
///////////////////////////////////////////////////////////////
/*
    Maintenance History
    ===================
    ver 1.2 - 09/23/99
      - modified the formats for output in display routine
      - eliminated includes for stdafx.h used when compiling
        with the MFC library.
      - created a C language implementation file, gaussel.c.
        This header works with both gaussel.cpp and gaussel.c.
    ver 1.1 - 10/19/1998
      - fixed bug in elimination process by adding elim(...) routine
        that divides by the pivot during elimination
      - transformed demonstration program into this modular form
      - cosmetic changes to comments
    ver 1.0 - 10/12/99
      - first release
*/

#include <cmath>
#include <cassert>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <direct.h>
#include <stdlib.h>
#include <math.h>
#include <string>

//#include "surface.h"

class gaussel
{
public :
	gaussel();
	~gaussel();
	long double** createMat();
	void GaussEl(long double **amat);
	void backSub(long double **amat);
	void sol(long double **amat, long double *vec);
	void destroyMat(long double **mat);
	void display(char *s, long double **amat);

private :

};

#endif